@@ -109,10 +109,16 @@ class Graph(metaclass=ContextMeta):
             return self._execute(cypher, parameters, backoff, limit)
 
     def execute(self, cypher, **payload):
-        if not self.write_allowed:
+        lower = cypher.lower()
+        if not self.write_allowed and ('create' in lower or 'merge' in lower or
+                                       'set' in lower or 'delete' in lower or 'detach' in lower):
             raise IOError(f"Write is not allowed, set `write=True` to permit writing.")
         d = _convert_datatypes(payload, nan2missing=True, none2missing=True)
-        return self._execute(cypher, d)
+        try:
+            return self._execute(cypher, d)
+        except IndexError:
+            raise ConnectionResetError(f"Py2neo dropped the connection because it was taking too long. "
+                                       f"Split up your query using batch_size=??")
 
     def output_for_debug(self, **payload):
         d = _convert_datatypes(payload, nan2missing=True, none2missing=True)
